@@ -6,7 +6,7 @@ import tensorflow as tf
 
 import qiznlp.common.utils as utils
 import qiznlp.common.train_helper as train_helper
-from qiznlp.run.run_base import Run_Model_Base
+from qiznlp.run.run_base import Run_Model_Base, check_and_update_param_of_model_pyfile
 
 curr_dir = os.path.dirname(os.path.realpath(__file__))
 sys.path.append(curr_dir + '/..')  # 添加上级目录即默认qiznlp根目录
@@ -42,7 +42,7 @@ class Run_Model_Mch(Run_Model_Base):
             self.tokenize = tokenize
         self.cut = lambda t: ' '.join(self.tokenize(t))
         self.token2id_dct = {
-            'word2id': utils.Any2Id.from_file(f'{curr_dir}/../data/mchword2id.dct', use_line_no=True),
+            'word2id': utils.Any2Id.from_file(f'{curr_dir}/../data/mch_word2id.dct', use_line_no=True),  # 自有数据
         }
         self.config = tf.ConfigProto(allow_soft_placement=True,
                                      gpu_options=tf.GPUOptions(allow_growth=True),
@@ -60,9 +60,9 @@ class Run_Model_Mch(Run_Model_Base):
             self.model = MCH_Model.from_pbmodel(pbmodel_dir, self.sess)
         else:
             with self.graph.as_default():
-                self.model = MCH_Model(model_name=self.model_name)
+                self.model = MCH_Model(model_name=self.model_name, run_model=self)
                 if self.use_hvd:
-                    self.model.optimizer._lr = self.model.optimizer._lr * hvd.size()  # 分布式训练大batch增大学习率
+                    self.model.optimizer._lr = self.model.optimizer._lr * self.hvd_size  # 分布式训练大batch增大学习率
                     self.model.hvd_optimizer = hvd.DistributedOptimizer(self.model.optimizer)
                     self.model.train_op = self.model.hvd_optimizer.minimize(self.model.loss, global_step=self.model.global_step)
                 self.sess.run(tf.global_variables_initializer())
@@ -89,26 +89,28 @@ class Run_Model_Mch(Run_Model_Base):
                                                feed_dict=feed_dict)
         return loss, accuracy, y_prob
 
-    def train(self, ckpt_dir, raw_data_file, preprocess_raw_data, batch_size=100):
+    def train(self, ckpt_dir, raw_data_file, preprocess_raw_data, batch_size = 100, save_data_prefix = None):
+        save_data_prefix = os.path.basename(ckpt_dir) if save_data_prefix is None else save_data_prefix
         train_epo_steps, dev_epo_steps, test_epo_steps, gen_feed_dict = self.prepare_data(conf.data_type,
                                                                                           raw_data_file,
                                                                                           preprocess_raw_data,
                                                                                           batch_size,
-                                                                                          save_data_prefix=os.path.basename(ckpt_dir),
+                                                                                          save_data_prefix=save_data_prefix,
                                                                                           )
         self.is_master = True
         if hasattr(self, 'hvd_rank') and self.hvd_rank != 0:  # 分布式训练且非master
             dev_epo_steps, test_epo_steps = None, None  # 不进行验证和测试
             self.is_master = False
 
-        # 字典大小对齐
-        assert all([self.model.conf.vocab_size == len(self.token2id_dct['word2id']),
-                    ]), f'{self.model.conf.vocab_size} != {len(self.token2id_dct["word2id"])}'
+        # 字典大小自动对齐
+        check_and_update_param_of_model_pyfile({
+            'vocab_size': (self.model.conf.vocab_size, len(self.token2id_dct['word2id'])),
+        }, self.model)
 
         train_info = {}
         for epo in range(1, 1 + conf.n_epochs):
             train_info[epo] = {}
-            
+
             # train
             time0 = time.time()
             epo_num_example = 0
@@ -192,6 +194,7 @@ class Run_Model_Mch(Run_Model_Base):
                 self.save(ckpt_dir=ckpt_dir, epo=epo, info_str=info_str)
 
             utils.obj2json(train_info, f'{ckpt_dir}/metrics.json')
+            print('=' * 40, end='\n')
             if conf.early_stop_patience:
                 if self.stop_training(conf.early_stop_patience, train_info, 'dev_acc'):
                     print('early stop training!')
@@ -218,6 +221,8 @@ class Run_Model_Mch(Run_Model_Base):
 
 def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
     """
+    # 处理自有数据函数模板
+    # file文件数据格式: 句子1\t句子2 (正样本对，框架自行负采样)
     # [filter] 过滤
     # [segment] 分词
     # [build vocab] 构造词典
@@ -241,6 +246,9 @@ def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
         # 读取分词好的数据
         items = utils.file2items(seg_file)
 
+    # 划分 不分测试集
+    train_items, dev_items = utils.split_file(items, ratio='19:1', shuffle=True, seed=1234)
+
     # 构造词典(option)
     need_to_rebuild = []
     for token2id_name in token2id_dct:
@@ -250,18 +258,17 @@ def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
 
     if need_to_rebuild:
         print(f'生成缺失词表文件...{need_to_rebuild}')
-        for item in items:
-            if 'word2id' in need_to_rebuild:
-                token2id_dct['word2id'].to_count(item[0].split(' '))
-                token2id_dct['word2id'].to_count(item[1].split(' '))
+        for items in [train_items, dev_items]:  # 字典只统计train和dev
+            for item in items:
+                if 'word2id' in need_to_rebuild:
+                    token2id_dct['word2id'].to_count(item[0].split(' '))
+                    token2id_dct['word2id'].to_count(item[1].split(' '))
         if 'word2id' in need_to_rebuild:
             token2id_dct['word2id'].rebuild_by_counter(restrict=['<pad>', '<unk>', '<eos>'], min_freq=5, max_vocab_size=30000)
-            token2id_dct['word2id'].save(f'{curr_dir}/../data/mchword2id.dct')
+            token2id_dct['word2id'].save(f'{curr_dir}/../data/mch_word2id.dct')
     else:
         print('使用已有词表文件...')
 
-    # 切分数据集
-    train_items, dev_items = utils.split_file(items, ratio='19:1', shuffle=True, seed=1234)
     # 负采样
     train_items = train_helper.gen_pos_neg_sample(train_items, sample_idx=1, num_neg_exm=4)
     dev_items = train_helper.gen_pos_neg_sample(dev_items, sample_idx=1, num_neg_exm=4)
@@ -269,14 +276,16 @@ def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
 
 
 if __name__ == '__main__':
-    os.environ['CUDA_VISIBLE_DEVICES'] = '0'
+    os.environ['CUDA_VISIBLE_DEVICES'] = '0'  # 使用CPU设为'-1'
 
-    # 自己数据集训练
-    rm_mch = Run_Model_Mch('esim')
+    rm_mch = Run_Model_Mch('esim')  # use ESIM match model
+
+    # 训练自有数据
     rm_mch.train('mch_ckpt_1', '../data/mch_example_data.txt', preprocess_raw_data=preprocess_raw_data, batch_size=512)  # train
-    rm_mch.restore('mch_ckpt_1')  # infer
-    import readline
 
+    # demo自有数据语义匹配模型
+    rm_mch.restore('mch_ckpt_1')  # for infer
+    import readline
     while True:
         inp = input('(use ||| to split) enter:')
         sent1, sent2 = inp.split('|||')
