@@ -6,7 +6,7 @@ import tensorflow as tf
 
 import qiznlp.common.utils as utils
 import qiznlp.common.train_helper as train_helper
-from qiznlp.run.run_base import Run_Model_Base
+from qiznlp.run.run_base import Run_Model_Base, check_and_update_param_of_model_pyfile
 
 curr_dir = os.path.dirname(os.path.realpath(__file__))
 sys.path.append(curr_dir + '/..')  # 添加上级目录即默认qiznlp根目录
@@ -30,23 +30,22 @@ conf = utils.dict2obj({
 })
 
 
-# multi-s1句子之间用$$$分割
-
-
 class Run_Model_MMCH(Run_Model_Base):
     def __init__(self, model_name, tokenize=None, pbmodel_dir=None, use_hvd=False):
         # 维护sess graph config saver
         self.model_name = model_name
         if tokenize is None:
             self.jieba = jieba.Tokenizer()
-            # self.jieba.load_userdict(f'{curr_dir}/segword.dct')
+            # self.jieba.load_userdict(f'{curr_dir}/../data/segword.dct')
             self.tokenize = lambda t: self.jieba.lcut(re.sub(r'\s+', '，', t))
         else:
             self.tokenize = tokenize
         self.cut = lambda t: ' '.join(self.tokenize(t))
         self.token2id_dct = {
-            'word2id': utils.Any2Id.from_file(f'{curr_dir}/../data/DBword2id.dct', use_line_no=True),
-            'char2id': utils.Any2Id.from_file(f'{curr_dir}/../data/DBchar2id.dct', use_line_no=True),
+            # 'word2id': utils.Any2Id.from_file(f'{curr_dir}/../data/mmch_word2id.dct', use_line_no=True),  # 自有数据
+            # 'word2id': utils.Any2Id.from_file(f'{curr_dir}/../data/mmch_char2id.dct', use_line_no=True),  # 自有数据
+            'word2id': utils.Any2Id.from_file(f'{curr_dir}/../data/DB_mmch_word2id.dct', use_line_no=True),  # 豆瓣多轮语料
+            'char2id': utils.Any2Id.from_file(f'{curr_dir}/../data/DB_mmch_char2id.dct', use_line_no=True),  # 豆瓣多轮语料
         }
         self.config = tf.ConfigProto(allow_soft_placement=True,
                                      gpu_options=tf.GPUOptions(allow_growth=True),
@@ -64,9 +63,9 @@ class Run_Model_MMCH(Run_Model_Base):
             self.model = MMCH_Model.from_pbmodel(pbmodel_dir, self.sess)
         else:
             with self.graph.as_default():
-                self.model = MMCH_Model(model_name=self.model_name)
+                self.model = MMCH_Model(model_name=self.model_name, run_model=self)
                 if self.use_hvd:
-                    self.model.optimizer._lr = self.model.optimizer._lr * hvd.size()  # 分布式训练大batch增大学习率
+                    self.model.optimizer._lr = self.model.optimizer._lr * self.hvd_size  # 分布式训练大batch增大学习率
                     self.model.hvd_optimizer = hvd.DistributedOptimizer(self.model.optimizer)
                     self.model.train_op = self.model.hvd_optimizer.minimize(self.model.loss, global_step=self.model.global_step)
                 self.sess.run(tf.global_variables_initializer())
@@ -93,25 +92,25 @@ class Run_Model_MMCH(Run_Model_Base):
                                                feed_dict=feed_dict)
         return loss, accuracy, y_prob
 
-    def train(self, ckpt_dir, raw_data_file, preprocess_raw_data, batch_size=100):
+    def train(self, ckpt_dir, raw_data_file, preprocess_raw_data, batch_size = 100, save_data_prefix = None):
+        save_data_prefix = os.path.basename(ckpt_dir) if save_data_prefix is None else save_data_prefix
         train_epo_steps, dev_epo_steps, test_epo_steps, gen_feed_dict = self.prepare_data(conf.data_type,
                                                                                           raw_data_file,
                                                                                           preprocess_raw_data,
                                                                                           batch_size,
-                                                                                          save_data_prefix=os.path.basename(ckpt_dir),
-                                                                                          update_txt=False
+                                                                                          save_data_prefix=save_data_prefix,
+                                                                                          update_txt=False,
                                                                                           )
         self.is_master = True
         if hasattr(self, 'hvd_rank') and self.hvd_rank != 0:  # 分布式训练且非master
             dev_epo_steps, test_epo_steps = None, None  # 不进行验证和测试
             self.is_master = False
 
-        # 字典大小对齐
-        assert all([self.model.conf.vocab_size == len(self.token2id_dct['word2id']),
-                    self.model.conf.char_vocab_size == len(self.token2id_dct['char2id']),
-                    ]), \
-            f'{self.model.conf.vocab_size} != {len(self.token2id_dct["word2id"])},' \
-            f'{self.model.conf.char_vocab_size} != {len(self.token2id_dct["char2id"])}'
+        # 字典大小自动对齐
+        check_and_update_param_of_model_pyfile({
+            'vocab_size': (self.model.conf.vocab_size, len(self.token2id_dct['word2id'])),
+            'char_vocab_size': (self.model.conf.char_vocab_size, len(self.token2id_dct['char2id'])),
+        }, self.model)
 
         train_info = {}
         for epo in range(1, 1 + conf.n_epochs):
@@ -200,6 +199,7 @@ class Run_Model_MMCH(Run_Model_Base):
                 self.save(ckpt_dir=ckpt_dir, epo=epo, info_str=info_str)
 
             utils.obj2json(train_info, f'{ckpt_dir}/metrics.json')
+            print('=' * 40, end='\n')
             if conf.early_stop_patience:
                 if self.stop_training(conf.early_stop_patience, train_info, 'dev_loss', greater_is_better=False):
                     print('early stop training!')
@@ -226,7 +226,8 @@ class Run_Model_MMCH(Run_Model_Base):
 
 def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
     """
-    # 自供数据 自定义数据处理函数模板
+    # 处理自有数据函数模板
+    # file文件数据格式: 多轮对话句子1\t多轮对话句子2\t...\t多轮对话句子n
     # [filter] 过滤
     # [segment] 分词
     # [build vocab] 构造词典
@@ -234,23 +235,35 @@ def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
     """
     seg_file = file.rsplit('.', 1)[0] + '_seg.txt'
     if not os.path.exists(seg_file):
-        items = utils.file2items(file)
+        sess_lst = utils.file2items(file)
         # 过滤
         # filter here
 
-        print('过滤后数据量', len(items))
+        print('过滤后数据量', len(sess_lst))
 
         # 分词
-        for i, item in enumerate(items):
-            item[0] = ' '.join(tokenize(item[0]))
-            item[1] = ' '.join(tokenize(item[1]))
-        utils.list2file(seg_file, items)
-        print('保存分词后数据成功', '数据量', len(items), seg_file)
+        for i, sess in enumerate(sess_lst):
+            # sess_lst[i] = [' '.join(s) for s in sess]  # 按字分
+            sess_lst[i] = [' '.join(tokenize(s)) for s in sess]  # 按词分
+        utils.list2file(seg_file, sess_lst)
+        print('保存分词后数据成功', '数据量', len(sess_lst), seg_file)
     else:
         # 读取分词好的数据
-        items = utils.file2items(seg_file)
+        sess_lst = utils.file2items(seg_file)
 
-    # 构造词典(option)
+    # 转为多轮格式 multi-turn之间用$$$分隔
+    items = []
+    for sess in sess_lst:
+        for i in range(1, len(sess)):
+            multi_src = '$$$'.join(sess[:i])
+            tgt = sess[i]
+            items.append([multi_src, tgt])
+    # items: [['w w w$$$w w', 'w w w'],...]
+
+    # 划分 不分测试集
+    train_items, dev_items = utils.split_file(items, ratio='19:1', shuffle=True, seed=1234)
+
+    # 构造词典(option) 字词联合
     need_to_rebuild = []
     for token2id_name in token2id_dct:
         if not token2id_dct[token2id_name]:
@@ -259,23 +272,38 @@ def preprocess_raw_data(file, tokenize, token2id_dct, **kwargs):
 
     if need_to_rebuild:
         print(f'生成缺失词表文件...{need_to_rebuild}')
-        for item in items:
-            if 'word2id' in need_to_rebuild:
-                token2id_dct['word2id'].to_count(item[0].split(' '))
-                token2id_dct['word2id'].to_count(item[1].split(' '))
+        for items in [train_items, dev_items]:  # 字典只统计train和dev
+            for item in items:
+                if 'word2id' in need_to_rebuild:
+                    for sent in item[0].split('$$$'):
+                        token2id_dct['word2id'].to_count(sent.split(' '))
+                    token2id_dct['word2id'].to_count(item[1].split(' '))
+                if 'char2id' in need_to_rebuild:
+                    for sent in item[0].split('$$$'):
+                        token2id_dct['char2id'].to_count(list(sent.replace(' ', '')))
+                    token2id_dct['char2id'].to_count(list(item[1].replace(' ', '')))
         if 'word2id' in need_to_rebuild:
-            token2id_dct['word2id'].rebuild_by_counter(restrict=['<pad>', '<unk>', '<eos>'], min_freq=5, max_vocab_size=30000)
-            token2id_dct['word2id'].save(f'{curr_dir}/word2id.dct')
+            token2id_dct['word2id'].rebuild_by_counter(restrict=['<pad>', '<unk>', '<eos>'], min_freq=1, max_vocab_size=30000)
+            token2id_dct['word2id'].save(f'{curr_dir}/../data/mmch_word2id.dct')
+        if 'char2id' in need_to_rebuild:
+            token2id_dct['char2id'].rebuild_by_counter(restrict=['<pad>', '<unk>', '<eos>'], min_freq=1, max_vocab_size=4000)
+            token2id_dct['char2id'].save(f'{curr_dir}/../data/mmch_char2id.dct')
     else:
         print('使用已有词表文件...')
 
-    # 切分数据集
-    train_items, dev_items = utils.split_file(items, ratio='19:1', shuffle=True, seed=1234)
+    # 负采样
+    train_items = train_helper.gen_pos_neg_sample(train_items, sample_idx=1, num_neg_exm=4)
+    dev_items = train_helper.gen_pos_neg_sample(dev_items, sample_idx=1, num_neg_exm=4)
     return train_items, dev_items, None
 
 
 def preprocess_common_dataset_Douban(file, tokenize, token2id_dct, **kwargs):
+    # 豆瓣多轮语料
+    # 对话数据分词,且生成词和字的字典word2id char2id
+    file = f'{curr_dir}/../data/Douban_Sess662.txt'
+
     def Doubanchange2items(file):
+        # 转为[multi_src, tgt]格式
         # 分词
         seg_file = file.rsplit('.', 1)[0] + '_seg.txt'
         if not os.path.exists(seg_file):
@@ -298,14 +326,12 @@ def preprocess_common_dataset_Douban(file, tokenize, token2id_dct, **kwargs):
                 exm_lst.append([multi_src, tgt])
         return exm_lst
 
-    # 转为[src, tgt]格式 且按字分
-    # Douban多轮对话数据分词,且生成词和字的字典word2id char2id
-    items = Doubanchange2items('../data/Douban_Sess662.txt')  # [['w w w$$$w w', 'w w w'],...]
+    items = Doubanchange2items(file)  # [['w w w$$$w w', 'w w w'],...]
 
     # 划分 不分测试集
     train_items, dev_items = utils.split_file(items, ratio='19:1', shuffle=True, seed=1234)
 
-    # 构造词典(option)
+    # 构造词典(option) 字词联合
     need_to_rebuild = []
     for token2id_name in token2id_dct:
         if not token2id_dct[token2id_name]:
@@ -326,10 +352,10 @@ def preprocess_common_dataset_Douban(file, tokenize, token2id_dct, **kwargs):
                     token2id_dct['char2id'].to_count(list(item[1].replace(' ', '')))
         if 'word2id' in need_to_rebuild:
             token2id_dct['word2id'].rebuild_by_counter(restrict=['<pad>', '<unk>', '<eos>'], min_freq=1, max_vocab_size=30000)
-            token2id_dct['word2id'].save(f'{curr_dir}/../data/DBword2id.dct')
+            token2id_dct['word2id'].save(f'{curr_dir}/../data/DB_mmch_word2id.dct')
         if 'char2id' in need_to_rebuild:
             token2id_dct['char2id'].rebuild_by_counter(restrict=['<pad>', '<unk>', '<eos>'], min_freq=1, max_vocab_size=4000)
-            token2id_dct['char2id'].save(f'{curr_dir}/../data/DBchar2id.dct')
+            token2id_dct['char2id'].save(f'{curr_dir}/../data/DB_mmch_char2id.dct')
     else:
         print('使用已有词表文件...')
 
@@ -340,17 +366,23 @@ def preprocess_common_dataset_Douban(file, tokenize, token2id_dct, **kwargs):
 
 
 if __name__ == '__main__':
-    os.environ['CUDA_VISIBLE_DEVICES'] = '0'
+    os.environ['CUDA_VISIBLE_DEVICES'] = '0'  # 使用CPU设为'-1'
 
-    # demo豆瓣多轮语料
-    rm_mmch = Run_Model_MMCH('MRFM')
+    # rm_mmch = Run_Model_MMCH('DAM')  # use DAM
+    rm_mmch = Run_Model_MMCH('MRFN')  # use MRFN
+
+    # 训练自有数据
+    # rm_mmch.train('multi_mch_ckpt_1', '../data/multi_mch_example_data.txt', preprocess_raw_data=preprocess_raw_data, batch_size=512)  # train
+
+    # 训练豆瓣多轮语料
     rm_mmch.train('multi_mch_ckpt_DB1', '', preprocess_raw_data=preprocess_common_dataset_Douban, batch_size=128)  # train
-    rm_mmch.restore('multi_mch_ckpt_DB1')  # infer
-    import readline
 
+    # demo豆瓣多轮检索式对话模型
+    rm_mmch.restore('multi_mch_ckpt_DB1')  # for infer
+    import readline
     while True:
         try:
-            inp = input('enter:($$$分隔多轮句子,|||分隔问句答句)')
+            inp = input('enter:($$$分隔多轮句子,|||分隔问句与答句)')
             sent1, sent2 = inp.split('|||')
             need_cut = False if ' ' in sent1 else True
             time0 = time.time()
@@ -361,6 +393,3 @@ if __name__ == '__main__':
             exit(0)
         except Exception as e:
             print(e)
-
-    # 自己数据集训练
-    # rm_mmch.train('multi_mch_ckpt_1', '../data/multi_mch_example_data.txt', preprocess_raw_data=preprocess_raw_data, batch_size=512)
